@@ -436,6 +436,11 @@ lav_data_full <- function(data          = NULL,          # data.frame
                                        complete.cases(data[,all.idx]))
                 nobs[[g]] <- length(case.idx[[g]])
                 norig[[g]] <- length(which(data[, group] == group.label[g]))
+            } else if(missing == "pairwise" && length(exo.idx) > 0L) {
+                case.idx[[g]] <- which(data[, group] == group.label[g] &
+                                       complete.cases(data[,exo.idx]))
+                nobs[[g]] <- length(case.idx[[g]])
+                norig[[g]] <- length(which(data[, group] == group.label[g]))    
             } else {
                 case.idx[[g]] <- which(data[, group] == group.label[g])
                 nobs[[g]] <- norig[[g]] <- length(case.idx[[g]])
@@ -445,6 +450,10 @@ lav_data_full <- function(data          = NULL,          # data.frame
                 case.idx[[g]] <- which(complete.cases(data[,all.idx]))
                 nobs[[g]] <- length(case.idx[[g]])
                 norig[[g]] <- nrow(data)
+            } else if(missing == "pairwise" && length(exo.idx) > 0L) {
+                case.idx[[g]] <- which(complete.cases(data[,exo.idx]))
+                nobs[[g]] <- length(case.idx[[g]])
+                norig[[g]] <- nrow(data)
             } else {
                 case.idx[[g]] <- 1:nrow(data)
                 nobs[[g]] <- norig[[g]] <- length(case.idx[[g]])
@@ -492,6 +501,9 @@ lav_data_full <- function(data          = NULL,          # data.frame
             # checking!
             if(length(Mp[[g]]$empty.idx) > 0L) {
                 X[[g]] <- X[[g]][-Mp[[g]]$empty.idx,,drop=FALSE]
+                if(length(exo.idx) > 0L) {
+                    eXo[[g]] <- eXo[[g]][-Mp[[g]]$empty.idx,,drop=FALSE]
+                }
                 if(warn) {
                     warning("lavaan WARNING: some cases are empty and will be removed:\n  ", paste(Mp[[g]]$empty.idx, collapse=" "))
                 }
